/*===============================================
// RS STARTERKIT
===============================================*/

'use strict'

const config = require('./config.json')
const pkg = require('./package.json')
const browserSync = require('browser-sync').create()
const concat = require('gulp-concat')
const del = require('del')
const gulp = require('gulp')
const sass = require('gulp-sass')
const sourcemaps = require('gulp-sourcemaps')
const uglifyjs = require('gulp-uglify')


/*===============================================
// TASKS
===============================================*/

// POSTCSS STACK

// SASS

gulp.task('sass', () => {
  return gulp.src(config.paths.source.scss + '/**/*.scss')
    .pipe(sourcemaps.init())
    .pipe(sass({
      includePaths: config.include.sass,
      outputStyle: 'expanded'
    }).on('error', sass.logError))
    .pipe(sourcemaps.write('.'))
    .pipe(gulp.dest(config.paths.public.css))
})

// JS

var pkgName = pkg.name
var pkgNameIdx = pkgName.indexOf('/')
var pkgNameSlice = pkgName.slice(pkgNameIdx + 1) 

gulp.task('js', () => {
  return gulp.src(config.include.js.concat(config.paths.source.js + '/**/*.js'))
    .pipe(uglifyjs())
    .pipe(sourcemaps.init())
    .pipe(concat(pkgNameSlice + '.min.js'))
    .pipe(sourcemaps.write('.'))
    .pipe(gulp.dest(config.paths.public.js))
})

// COPY

gulp.task('copy:images', () => {
  return gulp.src(config.paths.source.images + '/**/*.*')
    .pipe(gulp.dest(config.paths.public.images))
})

gulp.task('copy:fonts', () => {
  return gulp.src(config.paths.source.fonts + '/**/*.*')
    .pipe(gulp.dest(config.paths.public.fonts))
})

gulp.task('copy:html', () => {
  return gulp.src(config.paths.source.root + '*.html')
    .pipe(gulp.dest(config.paths.public.root))
})

// COPY ALL

gulp.task('copy', gulp.series(
  gulp.parallel(
    'copy:images',
    'copy:fonts',
    'copy:html'
  ),
  function(done){
    done()
  })
)

// DELETE

gulp.task('delete:all', () => {
  return del(config.paths.public.root)
})

/*===============================================
// SERVER
===============================================*/

function reload() {
  browserSync.reload()
}

gulp.task('serve', done => browserSync.init({
  server: {
    baseDir: config.paths.public.root
  },
  notify: {
    styles: [
      'display: none',
      'padding: 6px 12px',
      'font-family: Helvetica, Arial, sans-serif',
      'font-size: 0.82rem',
      'position: fixed',
      'z-index: 99999',
      'top: 0px',
      'left: 0px',
      'background-color: rgba(0, 0, 0, .8)',
      'margin: 0',
      'color: white'
    ]
  }
}, () => {
  done()
}))

/*===============================================
// WATCHERS
===============================================*/

function watch() {
  gulp.watch(config.paths.source.scss + '/**/*.scss', {
    awaitWriteFinish: true
  }).on('change', gulp.series('sass', reload))

  gulp.watch(config.paths.source.js + '/**/*.js', {
    awaitWriteFinish: true
  }).on('change', gulp.series('js', reload))

  gulp.watch(config.paths.source.root + '*.html', {
    awaitWriteFinish: true
  }).on('change', gulp.series('copy:html', reload))
}

/*===============================================
// GLOBAL TASKS
===============================================*/

gulp.task('devel', gulp.series('delete:all', 'sass', 'js', 'copy', 'serve', watch))
